import ddf.minim.*;
import ddf.minim.analysis.*;
import processing.serial.*;
import cc.arduino.*;

Arduino arduino;
//3-blue, 5-blue, 6-blue, 9-blue, 10-green, 11-white, 12-red, 13-orange
//Set these in the order of frequency - 0th pin is the lowest frequency,
//while the final pin is the highest frequency
int[] ledPins = {13,12,11,10,9,8,7,6,5,4,3,2,}; 
int[] lastFired = new int[ledPins.length];

//Change these to mess with the flashing rates
//Sensitivity is the shortest possible interval between beats
//minTimeOn is the minimum time an LED can be on
int sensitivity = 200;
int minTimeOn = 200;

String mode;
String source;

Minim minim;
AudioInput in;
AudioPlayer song;
BeatDetect beat;

//Used to stop flashing if the only signal on the line is random noise
boolean hasInput = false;
float tol = 0.005;

void setup(){

  //Uncomment the mode/source pair for the desired input
  
  //Shoutcast radio stream
  //mode = "radio";
  //source = "http://scfire-ntc-aa05.stream.aol.com:80/stream/1018";
  
  //mode = "file";
  //source = "/path/to/mp3";
  
  mode = "mic";
  source = "";
  
  size(512, 200, P2D);
    
  minim = new Minim(this);
  arduino = new Arduino(this, Arduino.list()[0]);
  
  for (int i = 0; i < ledPins.length; i++){
    arduino.pinMode(ledPins[i], Arduino.OUTPUT);
  }
  
  minim = new Minim(this);
  
  if (mode == "file" || mode == "radio"){
    song = minim.loadFile(source, 2048);
    song.play();
    beat = new BeatDetect(song.bufferSize(), song.sampleRate());
    beat.setSensitivity(sensitivity);
  } else if (mode == "mic"){
    in = minim.getLineIn(Minim.STEREO, 2048);
    beat = new BeatDetect(in.bufferSize(), in.sampleRate());
    beat.setSensitivity(sensitivity);
  }
}

void draw(){
  if (mode == "file" || mode == "radio"){
    beat.detect(song.mix);
    drawWaveForm((AudioSource)song);
  } else if (mode == "mic"){
    beat.detect(in.mix); 
    drawWaveForm((AudioSource)in);
  }
  
  if (hasInput){ //hasInput is set within drawWaveForm
    for (int i=0; i<ledPins.length; i++){
      if ( beat.isRange( i+1, i+1, 1) ){
        arduino.digitalWrite(ledPins[i], Arduino.HIGH);
        lastFired[i] = millis();
      } else {
        if ((millis() - lastFired[i]) > minTimeOn){
          arduino.digitalWrite(ledPins[i], Arduino.LOW);
        }
      }
    } 
  }
}  //End draw method

//Display the input waveform
//This method sets 'hasInput' - if any sample in the signal has a value
//larger than 'tol,' there is a signal and the lights should flash.
//Otherwise, only noise is present and the lights should stay off.
void drawWaveForm(AudioSource src){
  background(0);
  stroke(255);

  hasInput = false;
  
  for(int i = 0; i < src.bufferSize() - 1; i++)
  {
    line(i, 50 + src.left.get(i)*50, i+1, 50 + src.left.get(i+1)*50);
    line(i, 150 + src.right.get(i)*50, i+1, 150 + src.right.get(i+1)*50);
    
    if (!hasInput && (abs(src.left.get(i)) > tol || abs(src.right.get(i)) > tol)){
      hasInput = true;
    }
  } 
}

void resetPins(){
  for (int i=0; i<ledPins.length; i++){
    arduino.digitalWrite(ledPins[i], Arduino.LOW);   
  } 
}

void stop(){
  resetPins();  
  if (mode == "mic"){
    in.close();
  }  
  minim.stop();
  super.stop();
}
